#define IR_SMALLD_NECx  //define what type of IR remote you have
#include <IRsmallDecoder.h> //include the IR remote library
#include <RedBot.h> // Include the SparkFun RedBot library for motor control
RedBotMotors motors; //tie in motors to the Redbot library

IRsmallDecoder irDecoder(3); // Initialize IR decoder in pin 3
irSmallD_t irData; //set up variable for IR data


// Pin locations for line following sensors
RedBotSensor left = RedBotSensor(A2);   // initialize a left sensor object on A2
RedBotSensor center = RedBotSensor(A3); // initialize a center sensor object on A3
RedBotSensor right = RedBotSensor(A4);  // initialize a right sensor object on A4
//declare pin locations for distance sensor
const int trigPin = 10;
const int echoPin = 11;
// Create RedBotSensor objects for each line following sensor
RedBotSensor sr(srPin);
RedBotSensor sl(slPin);
RedBotSensor sc(scPin);

int i = 0;
// Variable for distance monitoring
float distance;
// Variable to store button press from IR remote
uint16_t x;

void setup() {
  //begin serial monitor
  Serial.begin(9600);
  //print to serial monitor to prompt button push
  Serial.println("Waiting for remote control signal");
  //claim pinmodes for ultrasonic distance sensor
  pinMode(trigPin,OUTPUT);
  pinMode(echoPin,INPUT);
}


void loop() {
  //if the IR reciever senses a button press run this loop
  if (irDecoder.dataAvailable(irData)) {
    x = irData.cmd; // Get the command from the IR remote store as x
    Serial.println(x); //print the button press
    delay(50); // Delay to prevent immediate repeat
    //start a switch case for different button presses
    switch (x) {
      case 16: // Line follower mode
      //run the line follower mode for 200 cycles
      for (i=0; i<200; i++){
        //run line following mode
        lineFollowerMode();}
        break;
      case 17: // Roomba mode
      //run roombah mode for 200 cycles
        for (i=0; i<200; i++){
          //run roombah mode
     roombaMode();}
        break;
      case 6: // Stop motors
        motors.brake();
        break;
      case 5: // Move forwards
        motors.drive(100); // Adjust speed as needed
        break;
      case 8: // Turn left
        motors.leftMotor(0); // Adjust speed as needed
        motors.rightMotor(200);
        break;
      case 10: // Turn right
        motors.rightMotor(0); // Adjust speed as needed
        motors.leftMotor(-200);
        break;
      case 13: // Move backwards
        motors.drive(-100); // Adjust speed as needed
        break;
      default:
        //if button isnt assigned print to serial monitor
        Serial.println("Button has no assigned action.");
        break;
    }
  }
}

void lineFollowerMode() {
  //if the line is over the center go straight
 if(center.read() > 850)
	{
    //our left motor went forward with negative prompts, otherwise they shgould both be positive 100
		motors.leftMotor(-100);
		motors.rightMotor(100); 
	}
	// if the line is under the right sensor, adjust relative speeds to turn to the right
	else if(right.read() > 850)
	{
		motors.leftMotor(-150);
		motors.rightMotor(50);
	}
	// if the line is under the left sensor, adjust relative speeds to turn to the left
	else if(left.read() > 850)
	{
		motors.leftMotor(-50);
		motors.rightMotor(150);
	}
  else {
    motors.coast();
  }
}

void roombaMode() {
  // Implement Roomba-like behavior
  // You might need to implement the getDistance() function or similar for obstacle detection
  //Serial.println("Key pressed is 2, Roombah mode");
    //read distance using getdistance function
     distance = getDistance();
     //if the distance is less than ten turn away
  while(distance < 10)
  {
    //go backwards for a second
    motors.rightMotor(-200);
    motors.leftMotor(-200);
    delay(1000);
    //turn right for half a second
    motors.rightMotor(200);
    motors.leftMotor(0);
    delay(500);
    //chack distance again
    distance = getDistance();
    
  }
  //while the distance is greater than 10 drive forwards
  while(distance > 10)
  {
    motors.drive(100);
    //continously check distance
    distance = getDistance();
    
}
}
float getDistance()
{
float echoTime; //variable to store the time it takes for a ping to bounce off an object
float calculatedDistance; //variable to store the distance calculated from the echo time
//send out an ultrasonic pulse that's 10ms long
digitalWrite(trigPin, HIGH);
delayMicroseconds(10);
digitalWrite(trigPin, LOW);
echoTime = pulseIn(echoPin, HIGH); //use the pulsein command to see how long it takes for the
//pulse to bounce back to the sensor
calculatedDistance = echoTime / 148.0; //calculate the distance of the object that reflected the pulse
Serial.println(calculatedDistance);
// (half the bounce time multiplied by the speed of sound)
return calculatedDistance; //send back the distance that was calculated
}



